import React, { useMemo } from 'react';
import { getReactStoreContext, initializers_get, setPageContext } from './context.js';
import { assert, removeFunctionsAndUndefined } from '../utils.js';
import { create as createZustand } from 'zustand';
import { devtools } from 'zustand/middleware';
import { cloneDeep, mergeWith } from 'lodash-es';
export default function VikeReactZustandWrapper({ pageContext, children }) {
    var _a;
    const initializers = initializers_get();
    const stores = useMemo(() => Object.entries(initializers).map(([key, initializer]) => {
        setPageContext(pageContext);
        const store = create(initializer);
        setPageContext(null);
        return [key, store];
    }), [initializers]);
    if (!stores.length) {
        return children;
    }
    const reactStoreContext = getReactStoreContext();
    assert(reactStoreContext);
    for (const [key, store] of stores) {
        // Trick to make import.meta.env.SSR work direclty on Node.js (without Vite)
        // @ts-expect-error
        (_a = import.meta).env ?? (_a.env = { SSR: true });
        if (import.meta.env.SSR) {
            pageContext._vikeReactZustand ?? (pageContext._vikeReactZustand = {});
            pageContext._vikeReactZustand = {
                ...pageContext._vikeReactZustand,
                [key]: removeFunctionsAndUndefined(store.getState())
            };
        }
        else if (!store.__hydrated__) {
            store.__hydrated__ = true;
            // TODO: remove lodash-es dependency and implement deep merging
            const merged = mergeWith(cloneDeep(store.getState()), pageContext._vikeReactZustand[key]);
            store.setState(merged, true);
        }
    }
    return React.createElement(reactStoreContext.Provider, { value: Object.fromEntries(stores) }, children);
}
function create(initializer) {
    return createZustand()(devtools(initializer));
}
