export { withPageContext };
import { getPageContext } from './renderer/context.js';
import { assert } from './utils.js';
/**
 * Middleware to make `pageContext` available to the store.
 *
 * Example usage:
 *
 * ```ts
 *
 * interface Store {
 *   user: {
 *     id: number
 *     firstName: string
 *   }
 * }
 *
 * const useStore = create<Store>()(
 *   withPageContext((pageContext) => (set, get) => ({
 *     user: pageContext.user
 *   }))
 * )
 * ```
 */
const withPageContext = (fn) => (set, get, store) => {
    const pageContext = getPageContext();
    assert(pageContext);
    return fn(pageContext)(set, get, store);
};
